*******************************************************************************************************************

*		               CANOPY RADIATION TRANSMISSION PROCESSES

*******************************************************************************************************************

********** PARTITION OF INCOMING SOLAR RADIATION INTO DIRECT AND DIFFUSE BEFORE IT HITS THE CANOPY *************       
c     Partition the incoming solar radiation into direct and diffuse components

	SUBROUTINE PSOLRAD(Qsi,atff,param, cf,                  ! Input Variables 
     *	                    Taufb,Taufd,Qsib,Qsid)          ! Output variables:


	REAL Qsi,cf,Taufb,Taufd,Qsib,Qsid,Taufc
	REAL param(*),as,bs,Lambda
			
      as     = param(28)			! Fraction of extraterrestaial radiation on cloudy day,Shuttleworth (1993)  
	bs     = param(29)			! (as+bs):Fraction of extraterrestaial radiation on clear day, Shuttleworth (1993) 
	Lambda = param(30)		    ! Ratio of diffuse atm radiation to direct for cler sky,from Dingman (1993)



       Taufc =max(atff,as+bs)     ! Mac atmosphereci transmittance for clear sky
   
	IF (cf.EQ.0) THEN 
		Taufb = Lambda*Taufc				 ! Direct solar radiation atm.transmittance
		Taufd = (1.-Lambda)*Taufc			 ! Diffuse solar radiation atm.transmittance
	ELSE IF (cf.EQ.1) THEN
		Taufb = 0.0						     
		Taufd = atff
	ELSE 
		Taufb = Lambda*(1.-cf)*(as+bs)
		Taufd = atff- Taufb
	END IF
    
C     Direct canopy solar radiation incident at the canopy 

		Qsib  = Taufb*Qsi/(Taufb+Taufd)
C	Diffuse canopy solar radiation incident at the canopy   
     
		Qsid  = Taufd*Qsi/(Taufb+Taufd)    
	
	RETURN
	END

C*********** TRANSMISSION OF DIRECT AND DIFFUSE SOLAR RADIATION THROUGH THE CANOPY ***************
c     Estimates the direct and diffuse solar radiation fractions transmitted through the canopy      
	SUBROUTINE  TRANSRADCAN (COSZEN,sitev,param,               ! Input variables : zenith angle, leaf parameters etc.
     *	         Betab,Betad,Taub,Taud)                        ! Output variables: Transmission and Reflection fractions

    
	REAL sitev(*),param(*),LAI,COSZEN,EXPI,G, kk
	REAL Taub,Taud,Taubh,Taudh, Beta, Betab, Betad


	Cc    = sitev(5)     ! Leaf Area Index 
	Hcan  = sitev(6)     ! Canopy height  
	LAI   = sitev(7)     ! Leaf Area Index  
	alpha = param(24)    ! 
      G     = param(26)    ! 

C      For open area where no vegetaion canopy is present                                       
	IF (LAI.EQ.0.)THEN 
	Taub = 1.
	Taud = 1.
      Betab = 0.
      Betad = 0.
 	ElSE 

C      Transmission without scattering 
      LAI   = Cc*LAI                      ! Effective leaf area index
      Rho   = LAI/Hcan                    ! leaf density per unit height of the canopy

c     Multiple scattering light penetration
	 kk    = sqrt(1-alpha)                             ! k-prime in the paper

	EXPI  = EXPINT(kk*G*LAI)                           ! Exponential integral function	

c      Deep Canopy Solution
c   Avoid divide by 0 error when COSZEN is 0
       if(COSZEN .LE. 0.)then
           Taubh=0.
       else
           Taubh =  EXP(- kk*G*Rho*Hcan/COSZEN)                 ! Transmission function for deep canopy : Direct
       endif
	 Taudh =  (1-kk*G*Rho*Hcan)*EXP(-kk*G*Rho*Hcan)+
     *	      (kk*G*Rho*Hcan)**2*EXPI                     ! Transmission function for deep canopy : Diffuse
     	 Beta   = (1-kk)/(1+kk)                                ! BetaPrime : reflection coefficient for infinitely deep canopy

c     Finite Canopy Solution
       Taub   = Taubh*(1-Beta**2)/(1-Beta**2*Taubh**2)     ! Direct fraction of radiation transmitted down through the canopy 
       Taud   = Taudh*(1-Beta**2)/(1-Beta**2*Taudh**2)         ! Diffuse fraction of radiation transmitted down through the canopy 
       Betab  = Beta*(1-Taubh**2) /(1-Beta**2*Taubh**2)    ! Direct fraction of radiation scattered up from the canopy 
       Betad  = Beta*(1-Taudh**2) /(1-Beta**2*Taudh**2)       ! Diffuse fraction of radiation scattered up from the canopy 

	ENDIF
	RETURN
	END

C****************** NET CANOPY & SUB CANOPY SOLAR RADIATION *********************************
c      Computes the net canopy and sub-canopy solar radiation
c      considering the multiple scatterings of radiation by the canopy and multile reflectios between the canopy and the snow surface
	SUBROUTINE NETSOLRAD(Ta,A,Betab,Betad,Wc,Taub,Taud,
     *	 Inmax,Qsib,Qsid,param,Fs,
     *                       Qsns,Qsnc ) !  Output: Qsns,Qsnc (Net subcanopy, canopy solar radiation) 
      
	REAL A,Betab,Betad,Wc,Inmax,Taub,Taud,Qsib,Qsid
	REAL param(*),Qsns, Qsnc


C	Net radiation at snow surface below canopy

		  f1b = (1-A)*Taub/(1-Betad*(1-Taud)*A) 
            f1d = (1-A)*Taud/(1-Betad*(1-Taud)*A)              
            Qsns =  f1b*Qsib + f1d*Qsid     
								                                                                    
		
	IF (Taub.EQ.1.0)THEN              ! (For LAI=0, Qsnc gives the numerical errors, so)
	Qsnc =0.0
	ELSE	                              
      
C     Net canopy solar radiation	

		 f3b = ((1-A)*Betab + A*Taub*Taud)/(1-Betad*(1-Taud)*A)
		 f3d = ((1-A)*Betad + A*Taud*Taud)/(1-Betad*(1-Taud)*A) 
		 f2b = 1- f1b-f3b  
		 f2d = 1- f1d-f3d
		 Qsnc = f2b*Qsib + f2d*Qsid  

	ENDIF  

	RETURN
	END

C****************** NET CANOPY & SUB CANOPY LONGWAVE RADIATION ******************************
c     Computes the net canopy and beneath canopy longwave radiation
	SUBROUTINE NETLONGRAD(RH,Ta,Tss,Tc,Tk,Fs,EmC,EmS,SBC,cf,sitev,Qli,
     *	                       param,
     *                                Qlis,Qlns,Qlnc )                     !  Output: Qsns,Qsnc 
 
	REAL RH,Ta,Tss,Tc,EmC,EmS,SBC,Fs,TaudL,sitev(*),LAI,G
	REAL EmA,ea, Qlis, Qlns,Qlnc,EXPI,kk,Beta,Betad,param(*)


	Cc    = sitev(5)     ! Leaf Area Index 
	Hcan  = sitev(6)     ! Canopy height  
	LAI   = sitev(7)     ! Leaf Area Index  
	alphaL= param(25)    ! 
      G     = param(26)    ! 	                              

c	ea   = SVPW(TA)*RH					            	! Air vapor pressure 
      TAK  = TA + TK
	Tssk = Tss +Tk  
      EmCS = EmC*(1-Fs)+EmS*Fs

	Tck  = Tc+Tk
      Qle  = EmS*SBC*Tssk**4

	IF (LAI.EQ.0.)THEN          !  For Open area where no vegetation caonpy is present
	  Qlcd = 0.
	  Qlcu = 0.
        Qlns = Qli*EmS -Qle                    ! To avoid numerical errors
        Qlnc = 0.
	ElSE 

C      Transmission without scattering      
	LAI      = Cc*LAI
      Rho      = LAI/Hcan  

c     Downward scattered 
	 kk    = sqrt(1-alphaL)
c     Upward scattered 
     	 Beta = (1-kk)/(1+kk)

	EXPI  = EXPINT(kk*G*LAI)                      	

c     Deep canopy solution
	 Taudh =  (1-kk*G*Rho*Hcan)*EXP(-kk*G*Rho*Hcan)+
     *	      (kk*G*Rho*Hcan)**2*EXPI                               ! Transmission function for deep canopy : Diffuse

c	 Finite canopy solution
         Taud   = (Taudh-Beta**2*Taudh) /(1-Beta**2*Taudh**2)         ! Transmission function for finite canopy depth
         Betad  = (Beta-Taudh*Beta*Taudh) /(1-Beta**2*Taudh**2)       !  Scatterd up  from top of the canopy at 0 depth  
c      Canopy emitted longwave radiation 
	  Qlcd = (1-Taud)*EmC*SBC*Tck**4
	  Qlcu = (1-Taud)*EmCs*SBC*Tck**4

CNet sub canopy longwave radiation
       Qlns = Taud*Qli*EmS + Ems*Qlcd-Qle+ (1-Taud)*(1.-1.0)*Qle

CNet canopy longwave radiation
      Qlnc = (((1-Taud)*1)+((1-Ems)*Taud*1))*Qli + (1-Taud)*Qle*1               ! Putting 1 for Emc
     * + (1-Taud)*(1-Ems)*1*Qlcu - Qlcu - Qlcd      

	ENDIF
      RETURN
      END

********************************************************************************************************************

*		               	TURBULENT TRANSFER PROCESSES

********************************************************************************************************************

C****************  BULK AERODYNAMIC AND CANOPY BOUNDARY LAYER RESISTANCES  *******************
C      Calculates resistances for the above and beneath canopy turbulent heat fluxes
 
	SUBROUTINE AeroRes(p,Wc,V,Ta,Tss,Tc,Fs,param,sitev,Tk,
     *             d,Z0c,Vz,Rc,Ra,Rbc,Rl,RKINc,RKINa,Rkinbc,Rkinl)         ! Output variables

	REAL z,d,k,Zm,Rimax,YcAge,Hcan,LAI,Vz,Ta,Tss,RKINsc,Kh,
     *	 Lbmean,Rkinca,Ru,Ra,Rc,Rs,Rca,Ri ,Rcastar, Rustar,ndecay,
     *   param(*),sitev(*), Tac, Dc,Rastar,Rkinaa,V

	data g   /9.81/        ! Gravitational acceleration (9.81 m/s^2)
      data k  /0.4/          ! Von Karmans constant (0.4)
      data hff /3600.0/      ! Factor to convert /s into /hr (3600)
      z     = param(5)       ! Nominal meas. height for air temp. and humidity (2m)
      zo    = param(6)       ! Surface aerodynamic roughness (m)
	Rimax = param(31)      ! Maximum value of Richardsion number for stability corretion
	fstab = param(19)      ! Stability correction control parameter 0 = no corrections, 1 = full corrections
      Wcoeff= param(32)      ! 

	Cc    = sitev(5)       ! Canopy Cover 
	Hcan  = sitev(6)       ! Canopy height  
	LAI   = sitev(7)       ! Leaf Area Index 

	Zm    = Hcan+z         ! observed wind speed above the canopy

      LAI = Cc*LAI
              
      IF(V.LE.0.)THEN
               Ra      = 0.
               Rc      = 0.
               Rs      = 0.
               Rbc     = 0.
               Rl      = 0.
        		 RKINc   = 0.                       
               RKINa   = 0.  
			 RKINbc  = 0.                       
               RKINl   = 0. 
			 Vz      = V 
			  !  If no wind, all resistances are zero; giving fluxes zero

      ELSE

        IF(LAI.EQ.0.)THEN      ! For open area
                       
          Vz      = V
          Rbc     = (1.0/(k**2*Vz)*(log(z/zo))**2.0 )*1/hff            ! sub layer snow resistance; hff: to convert resistance second to hour

	          Ri = g*(Ta-Tss)*Z /(Vz**2.0*(0.5*(Ta+Tss)+273.15))     
                                                    
	    		IF (Ri.GT.Rimax) THEN                                 
				Ri= Rimax
                  ENDIF                                               	    		
               
             		IF (Ri.GT.0.) THEN
					Rbcstar = Rbc/(1-5*Ri)**2      ! Bulk
				ELSE
      				Rbcstar = Rbc/((1-5*Ri)**0.75)
 		    	ENDIF

             RKINbc  = 1.0/Rbcstar                     ! Bulk conductance
	  ELSE


      CALL WINDTRANab(V,Zm,param,sitev,
     *                        Vstar,Vh,d,z0c, Vz,Vc )               ! Output wind speed within canopy at height 2 m           
      
C      Aerodynamic Resistances for Canopy
          ndecay =   Wcoeff*LAI              !(Wcoeff=0.5 for TWDEF, 0.6 for Niwot) ! wind speed decay coefficient inside canopy 

			Kh = k**2.0*V*(Hcan-d)/(log((Zm-d)/Z0c))
           
C        Above canopy aerodynamic resistance 
			Ra = (1.0* 1.0/(k**2.0*V)*log((Zm-d)/(z0c))*log((Zm-d)/         ! Increased ten times (Calder 1990, Lundberg and Halldin 1994)
     *		 (Hcan-d))+ Hcan/(Kh*ndecay)*
     *           (exp(ndecay-ndecay*(d+z0c)/Hcan)-1) )*1/hff

C        Below canopy aerodynamic resistance 
			Rc =(Hcan*exp(ndecay)/(Kh*ndecay)*(exp(-ndecay*z/Hcan)
     *		      -exp(-ndecay*(d+z0c)/Hcan))) *1/hff

			Rs = (1.0/(k**2*Vz)*(log(z/zo))**2.0)*1/hff	               
              Rc = Rc+ Rs                                                     ! Total below canopy resistance

C        Bulk aerodynamic resistance (used when there is no snow in the canopy)
		   Rbc = Rc+Ra                                               ! Total aerodynamic resistance from ground to above canopy used when there is no snow on canopy
                 
C     Boundary layer resistance of canopy(Rl)
        Dc      = .04    	                                             ! Deckinson et.al (1986);Bonan(1991): Characterstics dimension (m) of leaves (average width) ! put in parameters later
        Lbmean  = 0.02/ndecay*sqrt(Vc/Dc)* (1-exp(-ndecay/2))               
	  Rl      = 1/(LAI*Lbmean)*1/hff 

C        Leaf boundary layer conductance: reciprocal of resistance
	     RKINl  = 1.0/Rl  
	    
C      Correction for Ra, Rc and Rac for stable and unstable condition

	          Ri = g*(Ta-Tss)*Z /(Vz**2.0*(0.5*(Ta+Tss)+273.15))    ! wind speed unit should not be changed.
                                                    
	    		IF (Ri.GT.Rimax) THEN                                 
				Ri= Rimax
	    		ENDIF
                 
             		IF (Ri.GT.0.) THEN
					Rastar  = Ra  !/((1-5*Ri)**2)       ! Above canopy
					Rcstar  = Rc /((1-5*Ri)**2)      ! Below canpy
					Rbcstar = Rbc !  /((1-5*Ri)**2)     ! Bulk
                                                                      ! Rlstar  = Rl/((1-5*Ri)**2)         ! leaf boundary layer
				ELSE
					Rastar  = Ra  !/(1-5*Ri)**0.75 
					Rcstar  = Rc /(1-5*Ri)**0.75              
      				Rbcstar = Rbc  ! /(1-5*Ri)**0.75
				                                                    !	Rlstar  = Rl/(1-5*Ri)**0.75 
 				ENDIF
  

C     Turbulent (heat and vapor) transfer coefficient (Conductance) ! 
               RKINa   = 1.0/Rastar              ! Above canopy
	 		 RKINc   = 1.0/Rcstar              ! Below canpy                  
               RKINbc  = 1.0/Rbcstar             ! Bulk
c	         RKINl   = 1.0/Rlstar              ! leaf boundary layer
   
      ENDIF 
	ENDIF           
	RETURN
	END

********************* WIND PROFILES ****************************************************************
C     Calculates wind at the 2 m above the surface and at the sink using the input of measured wind at 2 m above the canopy

	SUBROUTINE WINDTRANab(V,Zm,param,sitev,
     *                        Vstar,Vh,d,z0c,Vz,Vc )               ! Out put wind speed within canopy at height 2 m

	REAL k,V,Vz,Hcan,LAI,Beta,Tac,RHc,Z,Zm,Z0c,d,Vstar,Vh
	REAL sitev(*),param(*) 

      data k  /0.4/           !  Von Karmans constant (0.4)
      Z      = param(5)       !  Nominal meas. height for air temp. and humidity (2m)
      Wcoeff= param(32)       ! 

	Cc     = sitev(5)       ! Canopy Cover 
	Hcan   = sitev(6)       ! Canopy height  
 	LAI    = sitev(7)       ! Leaf Area Index 
	Ycage  = sitev(9)      ! Parameters for wind speed transformation

      LAI = Cc*LAI
      
C     Roughness length (z0c) and zero plane displacement(d) for canopy [Periera, 1982]

      IF(V.LE.0.)THEN
	   Vz= 0.
	   Vc= 0.
	ELSE

	 d = Hcan*(0.05 + (LAI**0.20)/2 + (YcAge-1)/20)
                            

	IF (LAI.LT.1) THEN
		z0c= 0.1*Hcan
	ELSE
		z0c = Hcan*(0.23 - (LAI**0.25)/10 - (YcAge-1)/67)
	END IF

	Vstar  = k*V/log((Zm-d)/z0c)       !Friction velocity
        
C	Wind speed at height,Z from the ground/snow surface below canopy (Exponential profile from canopy height to below canopy)

      Vh  = 1./k*Vstar*log((Hcan-d)/z0c)  !Wind speed at the height of canopy (Logarithmic profile above canopy to canopy)
	  
	   Vz   = 1.0* Vh*exp(-Wcoeff*LAI*(1-z/Hcan)**1)
	   Vc   = 1.0* Vh*exp(-Wcoeff*LAI*(1-(d+Z0c)/Hcan)**1)     !To estimate canopy boundary layer conductance
c	   Vc   = 1.0* Vh*exp(-Wcoeff*LAI*(1-6./Hcan)**Beta)       

	ENDIF
	                          
	RETURN
	END

************************ TURBULENT FLUXES (ABOVE AND BELOW THE CANOPY *******************************

C     Calculates the turbulent heat fluxes (sensible and latent
C     heat fluxes) and condensation/sublimation.

      SUBROUTINE TURBFLUX(Ws,Wc,A,TK,Tc,Ta,Tss,RH,V,EA,p,param,sitev,
     *	              d,Z0c,Vz,RKINc,RKINbc,Tac,Fs,Ess,Esc,                       ! Output variables
     *                     QHc,QEc,Ec,QHs,QEs,Es,QH,QE,E)          
                                                      
      REAL Wc,RHOA,ESc,k,Apr, param(*),sitev(*),A,EA, LAI, EHo, EEo
      REAL QHc,QEc,Ec,QHs,QEs,Es,QH,QE,E
    

      z     =   param(5)      !  Nominal meas. height for air temp. and humidity (2m)
      zo    =   param(6)      !  Surface aerodynamic roughness (m)
      fstab =   param(19)     !  Stability correction control parameter 0 = no corrections, 1 = full corrections

      APr   =   sitev(2)     !  Atmospheric Pressure (Pa)
      Cc    =   sitev(5)      ! Canopy Coverage
	LAI   =   sitev(7)       ! Leaf Area Index 

	data Rag /287.0/     !  Ideal Gas constant for dry air (287 J/kg/K) (name changed)
      data k  /0.4/        !  Von Karmans constant (0.4)
      data hff /3600.0/    !  Factor to convert /s into /hr (3600)
      data rhow /1000.0/   !  Density of Water (1000 kg/m^3)
      data g    /9.81/     !  Gravitational acceleration (9.81 m/s^2)
      data cp /1.005/      !  Air Heat Capacity (1.005 KJ/kg/K)
      data hf /333.5/      !  Heat of fusion (333.5 KJ/kg)
      data hneu /2834.0/   !  Heat of Vaporization (Ice to Vapor, 2834 KJ/kg)

      data tol,nitermax/0.001,20/   
 
    	Tak   = Ta+Tk
      Tck   = Tc+Tk

	RHOA   = APr/(RAg*(TAK))	  ! RHOA in kg/m3,  APr: Atm Press
	Ea     = SVPW(Ta)*RH          ! Actual vapor pressure sorrounding canopy

c     Wind less coefficient:
       EHoA  = 0.0       ! for sensibleheat flux
	 EHoC  = 0.0
	 EHoS  = 0.0
	 EEoA  = 0.0       ! for latent heat flux
       EEoC  = 0.0
       EEoS  = 0.0

 	CALL AeroRes(p,Wc,V,Ta,Tss,Tc,Fs,param,sitev,Tk,
     *             d,Z0c,Vz,Rc,Ra,Rbc,Rl,RKINc,RKINa,Rkinbc,Rkinl)         ! Output variables
       

	IF (V.LE.0.) THEN
		   QHc  = 0.0
		   QEc  = 0.
		   Ec   = 0.
		   QHs  = 0.0
		   QEs  = 0.
		   Es   = 0.
             Tac  = Ta        ! Though We don't need Tac when V=0.
		
	ELSE

		IF (LAI.EQ.0.)THEN
						        
c     Flux estimation from the open areas when no canopy is present
			QHs = (RHOA*cp*RKINbc+EHos)* (TA-TSs)                        !	0.0 added for zero wind condition  
			QEs = (0.622*hneu/(RAg*(TAK))*RKINbc+EEos)*(EA-ESs) 
			Es  = -QEs/(RHOW*HNEU)                                       ! Es in  m/hr
			QHc = 0.
			QEc = 0.
			Ec  = 0.

		ELSE 

		   Tac = (Tc*Rkinl + Tss*Rkinc + Ta*Rkina)
     *		     	/(1*Rkina + 1*Rkinl + 1*Rkinc)
		   Eac = (ESc*Rkinl + ESs*Rkinc + Ea*Rkina)
     *		     	/(1*Rkina + 1*Rkinl + 1*Rkinc)
             Tack    = Tac+Tk   
    	       RHOAc   =  APr/(RAg*(TAcK))	              
                 
c	 Flux from canopy
		   QHc  = (RHOAc*cp*RKINl+EHoC)*(Tac-Tc) 
			       
             IF (Wc.EQ.0.and. p.EQ.0.)THEN 
                      QEc = 0.                          ! No latent heat flux when there is no snow in the caanopy
	                Ec  = 0.
		   ELSE	
               QEc  = (0.622*hneu/(RAg*(TAcK))*RKINl+ EEoC)*(EAc-ESc)*Fs
               Ec   = -QEc/(RHOW*HNEU)   
             ENDIF 			
c     Flux from snow at ground
			QHs  =(RHOAc*cp*RKINc +EHoS)* (Tac-TSs)                  !	EHoS added for zero wind condition [Jordan et al.(1999); Koivusalo (2002); Herrero et al.(2009) ] 
			QEs  =(0.622*hneu/(RAg*(TAcK))*RKINc+ EEoS)*(EAc-ESs) 
			Es   =- QEs/(RHOW*HNEU)
	ENDIF
c	Total flux
			QH  =  QHs + QHc 
			QE  =  QEs + QEc 
			 E  =  Es  + Ec  
	ENDIF             
      RETURN
      END

************************** RATE OF INTERCEPTION AND MAX INTERCEPTION ***********************
C     Calculates amount of snow intercepted by the canopy

	SUBROUTINE INTERCEPT(Ta,LAI,p,Wc,dt,Inmax,param,sitev,
     *	                            ieff,Ur,int)    ! Output variables

	REAL p,LAI,CC,Inmax,ieff, Rhofs, Sbar,S,int,dt,Ur 
	REAL Intma 
	REAL param(*),sitev(*)
	DOUBLE PRECISION Uc

    	Uc    = param(27)		    ! Unloading rate coefficient (Per hour) (Hedstrom and pomeroy, 1998) 
	Cc    = sitev(5)            ! Canopy Coverage

	IF(LAI.EQ.0.) 	CC=0.

C	Interception rate (di/dt)
      IF(p.EQ.0.) THEN
	    int  = 0.0
          ieff = 0.0
      ELSE
          Intma=0.
       IF (Wc.LT.Inmax) THEN
	    int  = CC*(1.-Wc*CC/(Inmax))*p
	    ieff = CC*(1.-Wc*CC/(Inmax))          ! Interception efficiency (di/dp) 
	    Intma =Inmax-Wc
		IF(int.GE.Intma) int= Intma

	  ELSE
	    int  = 0.0
          ieff = 0.0
	  END IF	

      END IF	

C	Unloading rate
	
	Ur = Uc*(Wc+(int*dt)/2)       ! Melt and Evap. are subtracted 
                                          ! half of the current interception is also considered for unloading
	RETURN	
	END

******************************   ENERGY ADVECTED BY RAIN   **********************************


C     Calculates the heat advected to the snowpack due to rain

      FUNCTION QPF(PR,TA,TO,PS,RHOW,HF,CW,CS)
      IF(TA.GT.TO) THEN
         TRAIN=TA
         TSNOW=TO													! TO : Freezing Temp (int.e.0C)
      ELSE
         TRAIN=TO
         TSNOW=TA
      ENDIF
      QPF=PR*RHOW*(HF+CW*(TRAIN-TO))+PS*RHOW*CS*(TSNOW-TO)
      RETURN
      END


****************************** PREHELP () ***************************************************
C      Routine to correct energy and mass fluxes when 
c      numerical overshoots dictate that W was changed in 
c      the calling routine - either because W went negative
c      or due to the liquid fraction being held constant.

       SUBROUTINE PREHELP(W1,W,DT,FM,FM1,fac,PS,PRAIN,E,RHOW,HF,Q,QM,MR,
     * qe,hneu)
       REAL MR,DT

       FM = (W1-W)/DT*fac-FM1
 
       MR = MAX( 0.0 , (PS + PRAIN - FM - E))
       E = Ps + Prain - FM - MR
C    Because melt rate changes the advected energy also changes.  Here
C     advected and melt energy are separated,
       QOTHER = Q + QM - QE
C     then the part due to melt recalculated
       QM = MR*RHOW*HF
C     then the part due to evaporation recalculated
       QE = -E*RHOW*HNEU
C     Energy recombined
       Q = QOTHER - QM + QE
       RETURN
       END

C****************** EXPONENTIAL INTEGRAL FUNCTION *****************************************
c     Computes the exponential integral function for the given value      
	FUNCTION EXPINT (LAI)
	REAL LAI
	DOUBLE PRECISION a0,a1,a2,a3,a4,a5,b1,b2,b3,b4
	IF (LAI.EQ.0)THEN
	    EXPINT=1.                          

	ELSEIF (LAI.LE.1.0) THEN
		a0=-.57721566
		a1=.99999193
		a2=-.24991055
		a3=.05519968
		a4=-.00976004
		a5=.00107857

		EXPINT = a0+a1*LAI+a2*LAI**2+a3*LAI**3+a4*LAI**4+a5*LAI**5 
     *		 -log(LAI)

	ELSE
		a1=8.5733287401
		a2=18.0590169730
		a3=8.6347637343
		a4=.2677737343
		b1=9.5733223454
		b2=25.6329561486
		b3=21.0996530827
		b4=3.9584969228

		EXPINT=(LAI**4+a1*LAI**3+a2*LAI**2+a3*LAI+a4)/
     *     	((LAI**4+b1*LAI**3+b2*LAI**2+b3*LAI+b4)*LAI*exp(LAI))

	END IF
	RETURN
      END

C      Reference Book:Handbook of mathematical functions with Formulas, Graphs, and Mathematical Tables
c      Edited by Milton Abramowitz and Irene A. Stegun, Volume 55, Issue 1972, page no. 231
c      Dover Publications, Inc, Mineola, New York.

*************************** SVP () ***********************************************************
C     Calculates the vapour pressure at a specified temperature over water or ice
c     depending upon temperature.  Temperature is celsius here.

      FUNCTION SVP(T)
      IF(t .ge. 0.)THEN
        svp=svpw(t)
      ELSE
        svp=svpi(t)

      ENDIF
	RETURN
	END



*************************** SVPW () **********************************************************
C     Calculates the vapour pressure at a specified temperature over water
C     using polynomial from Lowe (1977).

      FUNCTION SVPW(T)

      SVPW=6.107799961 + t * (0.4436518521 + t * (0.01428945805 + 
     & t * (0.0002650648471 + t * (3.031240936e-06 + t * 
     & (2.034080948e-08 + t * 6.136820929e-11)))))
      SVPW=SVPW*100											! convert from mb to Pa
      RETURN
      END



*************************** SVPI () *********************************************************
C     Calculates the vapour pressure at a specified temperature over ice.
C     using polynomial from Lowe (1977).

      FUNCTION SVPI(T)
      SVPI=6.109177956 + t * (0.503469897 + t * (0.01886013408 + 
     & t * (0.0004176223716 + t * (5.82472028e-06 + t * 
     &   (4.838803174e-08 + t * 1.838826904e-10)))))
      SVPI=SVPI*100											! convert from mb to Pa
      RETURN
      END


**********************************************************************************************
c     Estimates reflection and scattering coefficient

       FUNCTION Tau1(Rho,G,h,COSZEN,kk)
       REAL Rho,G,h,COSZEN,kk, Tau1

		IF (h.EQ.0.) THEN
		  Tau1=1.         ! To avoid computational error when coszen =0
		ELSE
	      IF (COSZEN.EQ. 0.) THEN
	       Tau1 = 0
            ELSE
	      Tau1  = EXP(- kk*G*Rho*h/COSZEN) 
		  ENDIF                  ! is the penetration function for deep canopy: direct
          ENDIF

       RETURN
       END
************************************************************************************************
       FUNCTION Tau2(Rho,G,h,kk,EXPI)
       REAL Rho,G,h,kk,EXPI,Tau2

 	     Tau2  = (1-kk*G*Rho*h)*EXP(-kk*G*Rho*h)+(kk*G*Rho*h)**2*EXPI     ! is the penetration function for deep canopy: diffuse

       RETURN
       END
************************************************************************************************





















































C      These sub routines and functions are not used





c***********************************   NEW  SHORTWAVE RADIATION MODEL-David   **************************  

	SUBROUTINE SOLRAD (Ta,A,Wc,coszen,
     *	                       Inmax,Qsib,Qsid,param,sitev,Fs,
     *                       Qsns,Qsnc ) !  Output: Qsns,Qsnc (Net subcanopy, canopy solar radiation) 

	REAL Ta,A,Wc,alpha,LAI,Hc,coszen,Qsib,Qsid,Fs
	REAL param(*),sitev(*),Qsns, Qsnc
      REAL Ah,Bh, Ch, Dh,Eh,Fh,Ao,Bo, Co, Do,Eo,Fo 
      REAL foo,f1,f2,f3,f4,f5,f6

	Ac    =   param(24)	
      Hcan  =   sitev(6)     !  
      LAI   =   sitev(7)     !    
	
	IF (LAI.EQ.0.) THEN
	 
	    Qsis = Qsib+Qsid               
        	Qsns = Qsis-A*Qsis 
      ELSE


	 ! Coefficients:
	  	alpha = .2  !Ac
	                                                 	!alpha = Ac*(1-Fs)+A*Fs
      	hc    = Hcan        ! Canopy height
      	Rho   = LAI/Hc
	    G     = 0.5
	    Rho   = Rho*G
          r1    = Rho*(sqrt(1-alpha))
          r2    = -r1

	IF (Qsib.EQ. 0.and.Qsid.EQ.0.) THEN
	 Qsns = 0.
	 Qsnc = 0.
       Qsnr = 0.
! --------------------------------------------------
! The coefficinet A, B, C, D, E , F and G at hieght y=hc and y=0.
       
      ELSE
	 IF (Qsib.EQ. 0.and.Qsid.NE.0.) THEN
          Qb   =  0.                     
	    foo  =  0.
	    Cc1  =  0.
	    Ah   =  0.
		Do   =  0.
	    Ao   =  0.
	    Dh   =  0.

	  GO TO 9
	  ENDIF

  
        Qb  =    Qsib*exp(-Rho*hc/coszen)      ! foo*Qsib                        ! directional radiance transmittance
	  foo =    Qb/Qsib
	  Cc1 = - Rho*alpha*Qsib*exp(-(Rho/COSZEN*hc))*(Rho/coszen) 

         IF (Cc1.EQ. 0.) THEN

		Ah   =   Qb*alpha/(1-alpha) 
		Do   =   Qb*alpha/(1-alpha) 
	    Ao   =   Qb*alpha/(1-alpha) 
	    Dh   =   Qb*alpha/(1-alpha) 
	   ELSE

		Ah   = Cc1*exp((Rho/coszen)*hc)*(1/((1-alpha)*coszen)+1)/
     *	    	((Rho/coszen)**2-r1**2)  +  Qb*alpha/(1-alpha) 

		Do   = Cc1*exp((Rho/coszen)*0)*(1/((1-alpha)*coszen)-1)/
     *		  ((Rho/coszen)**2-r1**2)   +  Qb*alpha/(1-alpha) 


	    Ao   = Cc1*exp((Rho/COSZEN)*0)*(1/((1-alpha)*COSZEN)+1)/
     *	      ((Rho/COSZEN)**2-r1**2)   +  Qb*alpha/(1-alpha) 




	    Dh   = Cc1*exp((Rho/COSZEN)*hc)*(1/((1-alpha)*COSZEN)-1)
     *		     /((Rho/COSZEN)**2-r1**2)   + Qb*alpha/(1-alpha) 


         ENDIF


 9     	Bh   = exp(r1*hc)*(r1/((1-alpha)*Rho)+1) 
		Ch   = exp(-r1*hc)*(r1/((1-alpha)*Rho)-1)

		Eo   = exp(r1*0)*(r1/((1-alpha)*Rho)-1) 
		Fo   = exp(-r1*0)*(r1/((1-alpha)*Rho)+1)

          Bo   = exp(r1*0)*(r1/((1-alpha)*Rho)+1) 
          Co   = exp(-r1*0)*(r1/((1-alpha)*Rho)-1)
 
	    Eh   = exp(r1*hc)*(r1/((1-alpha)*Rho)-1)
	    Fh   = exp(-r1*hc)*(r1/((1-alpha)*Rho)+1) 


!FOR DIRECT RADIATION 
c################################ FIRST CASE #############
! # Ibo=/0 , at y = yo,(yo=h),  Ido=0

        IF (Qsib.EQ.0.) GO TO 7

          C1b  = (Ah/Ch-Do/Fo)/((Eo/Fo)-(Bh/Ch))
          C2b  = (Do+Eo*C1b)/Fo
                                   !or
          C2b  = (Ah+Bh*C1b)/Ch

              Qdb  =  (Ao+Bo*C1b-Co*C2b)/2     ! f2* Qsib              ! (from direct radiation)
			Uob  = (Dh+Eh*C1b-Fh*C2b)/2      ! f1*Qsib               ! (from direct radiation)
			
		
				f2 = Qdb/Qsib
				f1 = uob/Qsib

!FOR DIFFUSE RADIATION 
!		######################################## SECOND CASE #############

7		C1d= - (2*Qsid/Ch)/((Eo/Fo)-(Bh/Ch))
		C2d= (Bh*C1d-2*Qsid)/Ch
		                           ! or
		C2d= Eo*C1d/Fo

			Qdd   = (Bo*C1d-Co*C2d)/2       ! f4* Qsid    ! (from diffuse radiation)
			Uod   = (Eh*C1d-Fh*C2d)/2       ! f3* Qsid    ! (from diffuse radiation)
				f4 = Qdd/Qsid
				f3 = uod/Qsid
!FOR REFLECTED RADIATION 
!		######################################## THIRD CASE #############

 		U = A*(Qb+Qdd+Qdb)

		C1u= (2*U/Fo)/((Eo/Fo)-(Bh/Ch))
		C2u= (Bh*C1u)/Ch
		                  !or
		C2u= (Eo*C1u-2*U)/Fo

			Qdu   =  (Bo*C1u-Co*C2u)/2   ! f6* Qsid        ! (from up scattering)
			Uou   =  (Eh*C1u-Fh*C2u)/2   ! f5* Qsid        ! (from up scattering)
				f6 = Qdu/U
				f5 = Uou/U

	 Qsns = (foo*Qsib+f2*Qsib+f4*Qsid)*(1-A+ f6*A/(1-f6*A))


C Fraction of radiation absorbed in surface snow,reflected back to atmosphere and absorbed in the canopy are:
	    shai= Qsib/(Qsib+Qsid)
	    F1s   = (foo*shai+f2*shai+f4*(1-shai))*(1-A+(f6*A)/(1-f6*A))

		F3s = (f1*shai+f3*(1-shai)+
     *	f5*A*(foo*shai+f2*shai+f4*(1-shai))*(1+(A*f6)/(1-A*f6)))
     		F2s =1-F1s-F3s

c NET RADIATION OBSERBED IN SURFACE SNOW AND CANPY ARE:
	 Qsns = F1s*(Qsib+Qsid)
	 Qsnc = F2s*(Qsib+Qsid)
       Qsnr = F3s*(Qsib+Qsid)

	ENDIF
      ENDIF
	RETURN
	END

c***********************************   NEW  LONGWAVE RADIATION MODEL- David   **************************  

	SUBROUTINE LONGRAD (RH,Ta,Tss,Tc,Tk,Fs,EmC,EmS,SBC,cf,Taud1,Qli,
     *	                      sitev,coszen,Qlis,Qlns,Qlnc ) !  Output: Qsns,Qsnc (Net subcanopy, canopy solar radiation) 

	REAL RH,Ta,Tss,Tc,EmC,EmS,SBC,Fs,LAI, sitev(*),U
	REAL EmA,ea, Qlis, Qlns,Qlnc,Qli, Qle,Qlcd
	REAL Ah,Bh, Ch, Dh,Eh,Fh,Ao,Bo, Co, Do,Eo,Fo 
	REAL f1,f2,f3,f4,f5,f6

      Hcan  =   sitev(6)     !  
      LAI   =   sitev(7)     ! 

     
c	ea   = SVPW(TA)*RH					            	! Air vapor pressure 
      TAK  = TA + TK
	Tssk = Tss +Tk  
   !   EmCS = EmC*(1-Fs)+EmS*Fs
      EmCS = EmC

    
	Tck  = Tc+Tk
	
	IF (LAI.EQ.0.) THEN
	 
	    Qlis = Qli           
        	Qlns = Qli-EmS*SBC*Tssk**4

      ELSE 

	 ! Coefficients:
	    alpha =  1-EmC
      	hc    =  Hcan        ! Canopy height
      	Rho   =  LAI/Hc
          G     = 0.5
	    Rho   = Rho*G
          r1    = Rho*(sqrt(1-alpha))
          r2    = -r1

	
       !alpha =0.03

! The coefficinet A, B, C, D, E , F and G at hieght y=hc and y=0.

c		Ah   = Qlcu/(1-alpha)
		Bh   = (exp(r1*hc)*(r1/((1-alpha)*Rho)+1)) 
		Ch   = exp(-r1*hc)*(r1/((1-alpha)*Rho)-1)

		!Do   =  Qlcd/(1-alpha)
		Eo   = (exp(r1*0)*(r1/((1-alpha)*Rho)-1)) 
		Fo   = exp(-r1*0)*(r1/((1-alpha)*Rho)+1)

c	  Ao   = Qlcd/(1-alpha)
        Bo   = (exp(r1*0)*(r1/((1-alpha)*Rho)+1)) 
        Co   = exp(-r1*0)*(r1/((1-alpha)*Rho)-1)

	  !Dh   = Qlcu/(1-alpha)
	  Eh   = (exp(r1*hc)*(r1/((1-alpha)*Rho)-1))
	  Fh   = exp(-r1*hc)*(r1/((1-alpha)*Rho)+1) 



!FOR INCOMING LONGWAVE RADIATION 
!		######################################## SECOND CASE #############

		C1d= - (2*Qli/Ch)/((Eo/Fo)-(Bh/Ch))
		C2d= (Bh*C1d-2*Qli)/Ch
		                           ! or
		C2d= Eo*C1d/Fo

			Qdi   = (Bo*C1d-Co*C2d)/2       ! f4* Qli   ! (from diffuse radiation)
			Uoi   = (Eh*C1d-Fh*C2d)/2       ! f3* Qli    ! (from diffuse radiation)
				f4 = Qdi/Qli
				f3 = Uoi/Qli

!FOR OUTGOING LONGWAVE RADIATION 
!		########################################  THIRD CASE #############

	    Qle  =  EmS*SBC*Tssk**4
		U    =  Qle
		C1u  = (2*U/Fo)/((Eo/Fo)-(Bh/Ch))
		C2u  = (Bh*C1u)/Ch
		                  !or
		C2u  = (Eo*C1u-2*U)/Fo

			Qdu   =  (Bo*C1u-Co*C2u)/2   ! f6* Qsid        ! (from up scattering)
			Uou   =  (Eh*C1u-Fh*C2u)/2   ! f5* Qsid        ! (from up scattering)
				f6 = Qdu/U
				f5 = Uou/U
c####################################### FIRST CASE
      !FOR RADIATION EMITTED BY THE CANOPY

	  Qlcd = (1-f5)*EmC*SBC*Tck**4   ! just to check the fraction that passes the canopy
	  Qlcu = (1-f5)*EmCs*SBC*Tck**4

	  Qlc = EmC*SBC*Tck**4
c  	  Qlcu = EmCs*SBC*Tck**4/Emcs

			Ah   = 2*Qlc /(1-alpha)
			Ao   = 2*Qlc /(1-alpha)
		    Do   = 2* Qlc /(1-alpha)
	        Dh   = 2*Qlc /(1-alpha)

          C1b  = (Ah/Ch-Do/Fo)/((Eo/Fo)-(Bh/Ch))
          C2b  = (Do+Eo*C1b)/Fo
                                   !or
          C2b  = (Ah+Bh*C1b)/Ch

              Qdb  =  (Ao+Bo*C1b-Co*C2b)/2     ! f2* Qsib               ! (from direct radiation)
			Uob  = (Dh+Eh*C1b-Fh*C2b)/2      ! f1*Qsib               ! (from direct radiation)
			f2 = Qdb/Qlc
			f1 = Uob/Qlc

C Fraction of radiation absorbed in surface snow,reflected back to atmosphere and absorbed in the canopy are:


c	    shai= Qsib/(Qsib+Qsid)

	  Qlis = (f4*Qli+f6*Qle+f1*Qlc)*(1+ f6*(1-EmS)/(1-f6*(1-EmS)))     ! F1l
        Qlns = (f4*Qli+f6*Qle+f1*Qlc)*(1+ f6*(1-EmS)/(1-f6*(1-EmS)))-Qle     
                                                        
	  Qlnr  = (f3*Qli+f5*Qle+f1*Qlc) +f5*(1-Ems)*
     *		  ((f4*Qli+f6*Qle+f1*Qlc)/(1-f6*(1-EmS)))                       !F3l


        Qlnc  = Qli + Qle - Qlis -Qlnr                                     ! F2l


      ENDIF
	RETURN
	END
c#############################################################################################
************************    CANOPY TURBULENT FLUXES    ***************************************

C     Calculates the turbulent heat fluxes (sensible and latent
C     heat fluxes) and condensation/sublimation.

!      SUBROUTINE TURBABOVECAN(Wc,A,TK,Tc,Ta,Rkinc,V,EA,param,sitev,
!     *	                     Apr,Fs,QHc,QEc,Ec)          ! Out put variables
                                                    
!      REAL Wc,RHOA,QHc,ESc,EAc,QEc,Ec,k,Apr, param(*),sitev(*)

!      z     =   param(5)      !  Nominal meas. height for air temp. and humidity (2m)
!      zo    =   param(6)      !  Surface aerodynamic roughness (m)
!      fstab =   param(19)     !  Stability correction control parameter 0 = no corrections, 1 = full corrections
!      Cc    =   sitev(6)      ! Canopy Coverage

!	data Rag /287.0/     !  Ideal Gas constant for dry air (287 J/kg/K) (name changed)
!      data k  /0.4/        !  Von Karmans constant (0.4)
!      data hff /3600.0/    !  Factor to convert /s into /hr (3600)
!      data rhow /1000.0/   !  Density of Water (1000 kg/m^3)
!      data g    /9.81/     !  Gravitational acceleration (9.81 m/s^2)
!      data cp /1.005/      !  Air Heat Capacity (1.005 KJ/kg/K)
!      data hf /333.5/      !  Heat of fusion (333.5 KJ/kg)
!      data hneu /2834.0/   !  Heat of Vaporization (Ice to Vapor, 2834 KJ/kg)


!	Tak  = Ta+Tk
!	     IF (Tc.EQ.0.0) THEN
!	       Tc =  Ta*(1-Fs)+Tc*Fs                        ! Canopy temp when snow is melting
!	     ENDIF                                    
!	Tck  = Tc+Tk

!		rkinc = RKINSTC(rkn,V,tak,tck,z,g,fstab,Zo,k,hff)
!		RHOA  = APr/(RAg*(TAK))	
!								         ! RHOA in kg/m3,  APr: Atm Press
!	 IF	(Wc. GT.0) THEN
!		QHc  =Cc*(RHOA*cp*rkinc)*(TA-Tc)    	
!		ESc  = SVPI(Tc)                       !  Ice ??????????????

!		QEc  = Cc*0.622*hneu/(RAg*(TAK))*rkinc*(EA-ESc)   ! Forest canopies are assumed to be saturated all the time
!		Ec   = -QEc/(RHOW*HNEU)                         ! E in m/hr

!	 ELSE 
!	 QHc = 0.
!	 QEc = 0.
!	 Ec  = 0.	
		                                 
!       ENDIF

!      RETURN
!      END

************************ TURBULENT FLUX FROM SNOW AT GROUND () **************************************************************

C     Calculates the turbulent heat fluxes (sensible and latent
C     heat fluxes) and condensation/sublimation.

!      SUBROUTINE TURBFLUXsc(V,Ta,Tss,Tk,param,sitev,z,g,EA,Zm,
!     *          d,Z0c,Vz,RKINsc,QH,QE,E)                     ! Output Fluxes from sub canopy
   
!      REAL RHOA,QH,ES,EA,QE,E,sitev(*)

!      data cp /1.005/      !  Air Heat Capacity (1.005 KJ/kg/K)
!      data Rag /287.0/     !  Ideal Gas constant for dry air (287 J/kg/K)
!      data rhow /1000.0/   !  Density of Water (1000 kg/m^3)
!      data hneu /2834.0/   !  Heat of Vaporization (Ice to Vapor, 2834 KJ/kg)
*      fstab = param(19)   !  Stability correction control parameter 0 = no corrections, 1 = full corrections
!      APr   = sitev(3)     !  Atmospheric Pressure (Pa)
!      Cc    = sitev(6)   ! Canopy Coverage  
	 
!      tak=ta+tk
!      tssk=tss+tk

!      CALL AeroRes(V,Ta,Tss,param,sitev,Tk,
!     *                      d,Z0c,Vz,RKINsc)         ! Output variables

!	CALL BOUNDRES(Ra, Rc, Rs,Ta, Tss, Tc,
!     *	                         Ru,Tac)            !Output boundary layer resistance and air temperature sorrounding canopy



 !     RHOA=Apr/(RAg*(TAK))

C     RHOA in kg/m3
!      QH=(1-Cc*0.)*(RHOA*cp*RKINsc+0.0)* (TA-TSs)    !	3.6 added for zero wind condition  
!      ES=SVPI(TSs)
!	QE=(1-Cc*0.)* 0.622*hneu/(RAg*(TAK))*RKINsc*(EA-ES) 
!      E=-QE/(RHOW*HNEU)
C     E in  m/hr

!      RETURN
!      END


**********************************************************************************************
c***********************************   NEW  SHORTWAVE RADIATION MODEL  **************************  
c     REF: Norman, J. M.(1979). Modeling the completer crop canopy. Modification of the Aerial Environment of Plants. 
c     (Ed. by B.J. Barfield and J.F.Gerber). American scociety of Agricultural Engineers, St Joseph, Michigan.


	SUBROUTINE NETSOLRAD2(Ta,A,Wc,Taub,Taub1,Taus1b,Taud,Taud1,Taus1d,
     *	                      Inmax,Qsib,Qsid,param,Fs, sitev,
     *                              Qsns,Qsnc )     !  Output: Qsns,Qsnc (Net subcanopy, canopy solar radiation) 

    
	REAL A,Wc,Inmax,Taub,Taub1,Taus1b,Taud,Taud1,Taus1d,Qsib,Qsid
	REAL param(*),Qsns, Qsnc, Fs
	REAL sitev(*)
	REAL Al(6)                 ! Albedo of each canopy layer
	REAL Qd(6)
	REAL Qb(6)
	REAL U(6)
	REAL Ala(6)                 ! Albedo of each canopy layer
	REAL Qbd(6)
	REAL Ub(6)
	data tol,nitermax/0.5,10/   ! 

 	LAI  =  sitev(8)        ! Leaf Area Index  
 	Cc   =  sitev(6)  

!	Open Part
           Qopen  = (1-Cc)*(Qsid+Qsib)

!     Shading part
         Qsid = Qsid*Cc
         Qsib = Qsib*Cc


  
      ! Canopy is divided into N number of layers 
        n=6
       Trc   =  .25              ! Leaf transmittance
       Rfc   =  .25               ! Leaf reflectance


	 Al(1) =  A      ! Snow albedo 1st layer at the bottom      

      
c     Calculate diffuse radiation albedo for each canopy layer with known bottom snow (ground) albedo  

	do  5 i=2,n

	 Al(i) = (((Trc*(1-Taud1)+Taud1)**2 -(Rfc*(1-Taud1))**2)*Al(i-1)
     *	 + Rfc*(1-Taud1))/(1-Rfc*(1-Taud1)*Al(i-1))
	
5      continue
 
C     All downward fluxes (Qd)
 
	Qd(n) = Qsid                             ! + Qsib           ! Downward Diffuse +Direct Radiation at the top for the iteration
  
  	do j=n-1,1,-1 
         Qd(j) = Qd(j+1)*((Trc*(1-Taud1)+Taud1)/(1-Rfc*(1-Taud1)*Al(j)))

C       All upward fluxes (U)  

	   U(n)  = Al(n)*Qd(n)      ! Upward Diffuse Radiation at the top
	   U(j) =  Al(j)*Qd(j)
      end do

C Now iterate for the better solution 
       ! Downward diffuse  radiation (Qd)
  
       Qb(n) = Qsib           ! Downward Direct Radiation at top layer
  
  	do j=n-1,1,-1 

	  ERr    = tol*2                           
        iter   =  0. 

 7      Qdlast = Qd(j) 
    	   Qb(j) = Taub1*Qb(j+1)    ! Direct radiation attenuation
        Qd(j)=Qd(j+1)* (Trc*(1-Taud1)+Taud1)+Al(j)*
     *	    Qd(j)*(Rfc*(1-Taud1))+Qb(j+1)*(1-Taub1)*Trc

           IF(ERr.gt.tol.and.iter.LT. nitermax)THEN 
            ERr    = abs(Qd(j) -  Qdlast)
		  iter   = iter+1
	       go to 7 
           ENDIF

       ! Upward diffuse radiation (U)
     
      U(j+1) = Al(j+1)*Qd(j+1)          ! This is iterated
c       U(n) = Al(j+1)*(Qsid+Qsib)          ! This is iterated	 
	 
	 ERr    = tol*2                           
       iter   =  0. 

 8     Ulast = U(j+1)
       Qb(j) = Taub1*Qb(j+1)    ! Direct radiation attenuation
       U(j+1) = Al(j)*Qd(j)* (Trc*(1-Taud1)+Taud1)+Al(j+1)*U(j+1)*
     *	    	(Rfc*(1-Taud1))+Qb(j+1)*(1-Taub1)*Rfc

           IF(ERr.gt.tol.and.iter.LT. nitermax)THEN 
            ERr    = abs(U(j+1) -  Ulast)
		  iter   = iter+1
	       go to 8 
           ENDIF
      end do 
	 	   
      U(1) = Al(1)*Qd(1) 

c######################################################################################
c     Direct radiation (Qb1) when reflected back from the snow surface takes place the multiple scatterig again
c     Albedo 

      Ala(n) =	Rfc

	do  3 i=n-1,2,-1 

	 Ala(i) = (((Trc*(1-Taud1)+Taud1)**2 -(Rfc*(1-Taud1))**2)*Ala(i+1)
     *	 + Rfc*(1-Taud1))/(1-Rfc*(1-Taud1)*Ala(i+1))
	
3      continue

C     All upward fluxes (Ub)
 
	Ub(1) = Al(1)*Qb(1)        ! First reflected direct radiation
  
  	do j=2,n 
      Ub(j) = Ub(j-1)*((Trc*(1-Taud1)+Taud1)/(1-Rfc*(1-Taud1)*Ala(j)))
C       All downward fluxes (Qdd)  

	   Qbd(1)  = Ala(1)*Ub(1)      ! Upward Diffuse Radiation at the top
	   Qbd(j) =  Ala(j)*Ub(j)
      end do
c###################################################################################

   
C	Net radiation at snow surface below canopy (shading)
		Qsns = Qd(1)-U(1) + Qb(1)*(1-Al(1))

           Qsns =Qsns+Qopen*(1-Al(1))     ! Including Q in Open fraction
 
C	Net radiation reflected back from the  canopy and ground
		Qsnr= U(n)  + Ub(5)                                         !+Qopen*Al(1)
		Qsnr = Qsnr+Qopen*Al(1)	                    
								    
C	Net radiation  canopy
		Qsnc = Qsid+Qsib - Qsnr-Qsns + Qopen

c	ENDIF
	RETURN
	END

c***********************************   NEW  LONGWAVE RADIATION MODEL  **************************  
c     REF: Norman, J. M.(1979). Modeling the completer crop canopy. Modification of the Aerial Environment of Plants. 
c     (Ed. by B.J. Barfield and J.F.Gerber). American scociety of Agricultural Engineers, St Joseph, Michigan.

	SUBROUTINE NETLONGRAD2(RH,Ta,Tss,Tc,Tk,Fs,EmC,EmS,SBC,cf,Taud1,
     *	                          Qli, sitev,
     *                         Qlis,Qlns,Qlnc )                     !  Output: Qsns,Qsnc 
 
	REAL RH,Ta,Tss,Tc,EmC,EmS,SBC,Fs
	REAL EmA,ea, Qlis, Qlns,Qlnc
	REAL Qli, Qle,f1i,f2i,f3i,f1e,f2e,f3e,f1cd,f2cd,f3cd


	REAL A,Wc,Inmax,Taub1,Taus1b,Taud,Taud1,Taus1d,Qsib,Qsid,LAI
	REAL sitev(*)
	REAL Al(6)                 ! Albedo of each canopy layer
	REAL Qlcu(6)
	REAL Qlcd(6)
	data tol,nitermax/0.5,10/   ! 
        
 	LAI   =  sitev(8)        ! Leaf Area Index  
 	Cc   =  sitev(6)   	    
	
c	ea   = SVPW(TA)*RH					            	! Air vapor pressure 
      TAK  = TA + TK
	Tssk = Tss +Tk  
      EmCS = EmC*(1-Fs)+EmS*Fs

	Tck  = Tc+Tk
   
!	Open Part
           Qlopen  = (1-Cc)*Qli

!     Shading part
          Qlsh = Qli*Cc
    

	  Qle  = EmS*SBC*Tssk**4
!	  Qlcd = (1-Taud1)*EmC*SBC*Tck**4
!	  Qlcu = (1-Taud1)*EmCs*SBC*Tck**4

  
      ! Canopy is divided into N number of layers 
        n=6
c       Trc   =  .25               ! Leaf transmittance
c       Rfc   =  .25               ! Leaf reflectance
    
C     All downward fluxes (Qd)
 
	Qlcd(n) = Qlsh                          ! + Qsib           ! Downward Diffuse +Direct Radiation at the top for the iteration
  
  	do j=n-1,1,-1 
         Qlcd(j) = Qlcd(j+1)*Taud1+
     *	         1./(n-1)*(1-Taud1)*EmC*SBC*Tck**4
	end do  
	   
	Qlcu(1) = 	  Cc*Qle     
  	do j=2,n	    
         Qlcu(j) = Qlcu(j-1)*Taud1+
     *              1./(n-1)*(1-Taud1)*EmCs*SBC*Tck**4   
      end do

c	EXPI=EXPINT(LAI/2)                             ! Exponential integral function               
c	Taud = (1-LAI/2)*EXP(-LAI/2)+(LAI**2)/4*EXPI
c	  Qlcdd = (1-Taud1)*EmC*SBC*Tck**4
c	  Qlcuu = (1-Taud1)*EmCs*SBC*Tck**4


CNet sub canopy longwave radiation
       Qlns = Qlcd(1)-Qlcu(1)+Qlopen*EmS- Qle*(1-Cc)
CNet reflected longwave radiation from canopy
      Qlnr = Qlcu(n)+Qlopen*(1-EmS)+ Qle*(1-Cc)
CNet canopy longwave radiation
      Qlnc = Qle + Qli -Qlns-Qlnr

	RETURN
	END

C****************** NET CANOPY & SUB CANOPY SOLAR RADIATION *********************************
	
c***********************************  SHORTWAVE RADIATION MODEL- TWO STREAM **************************  
c     REF: P.J Seller (1985). Canopy reflectance, photosynthesis and transpiration 

	SUBROUTINE NETSOLRAD4(Ta,A,Wc,Taub,Taub1,Taus1b,Taud,Taud1,Taus1d,
     *	                    Inmax,Qsib,Qsid,param,Fs, sitev,coszen,
     *                             Qsns,Qsnc )     !  Output: Qsns,Qsnc (Net subcanopy, canopy solar radiation) 
    
	REAL A,Wc,Inmax,Taub,Taub1,Taus1b,Taud,Taud1,Taus1d,Qsib,Qsid
	REAL param(*),Qsns, Qsnc, Fs, LAI
	REAL sitev(*),coszen
	REAL W, K, Mubar


 	LAI  =  sitev(8)        ! Leaf Area Index  
 	Cc   =  sitev(6)  
      LAI  = Cc*LAI



      IF (coszen.EQ. 0.) THEN
	Qsns =0.
	Qsnr= 0.
	Qsnc =0.
	go to 6
	ENDIF


!      coszen =0.5

c	single scattering albedo as(Mu) for the spherically distributed leaves

c      New paramters
      W    = 0.5          ! Scattering coefficient
      Beta = 0.34         ! Reflectance for diffuse radiation
c	Betao = Beta            ! Reflectance for direct radiation
      Tr   = .11	                   ! Transmission for for the diffuse radaiation 
      K   =  .5/coszen
	Mubar = 1.  
	
	          !
       as    =  w/2*(1-Coszen*log((Coszen+1)/coszen))
	 Betao = (1+Mubar*k)*as/(w*Mubar*K)



c     Fraction of downward and upward diffuse radiation from direct radiation

      b = (1-(1-Beta)*W)
	c = W*Beta
	d = W*Mubar*K*Betao
	f = W* Mubar*K*(1-Betao)
	h = ((b**2-c**2)**0.5)/Mubar

	Sigma = (Mubar*k)**2+c**2-b**2

	u1 = b-c/A
	u2 = b-c*A
	u3 = f+c*A
	S1 = exp(-h*LAI)
	S2 = exp(-k*LAI)
	p1 = b+Mubar*h
	p2 = b-Mubar*h
	p3 = b+Mubar*k
	p4 = b-Mubar*k



	D1  = p1*(u1-Mubar*h)/S1-p2*(u1+Mubar*h)*S1
	D2  = (u2+Mubar*h)/S1-(u2-Mubar*h)*S1

      h1  = -d*p4-c*f
	h2   = 1/D1*((d-h1/Sigma*p3)*(u1-Mubar*h)*1/S1-
     *	   p2*(d-c-h1/Sigma*(u1+Mubar*K))*S2)

	h3   = -1/D1*((d-h1/Sigma*p3)*(u1+Mubar*h)*S1-
     *	   p1*(d-c-h1/Sigma*(u1+Mubar*K))*S2)

	h4   =  f*p3 -c*d
	
	h5   = - 1/D2*(h4/Sigma*(u2+Mubar*h)*1/S1+
     *	   (u3-h4/Sigma*(u2-Mubar*K))*S2)

      h6   = 1/D2*(h4/Sigma*(u2-Mubar*h)*S1+
     *	   (u3-h4/Sigma*(u2-Mubar*K))*S2) 


      FUb = h1*exp(-k*LAI)/sigma +h2*exp(-h*LAI)+h3*exp(h*LAI)
      FQb = h4*exp(-k*LAI)/sigma +h5*exp(-h*LAI)+h6*exp(h*LAI)

	 
C     From diffuse radiation  
    
      h7  = c/D1*(u1-Mubar*h)* 1/s1
      h8  = -c/D1*(u1+Mubar*h)* s1      
	h9  = 1/D2*(u2+Mubar*h)* 1/s1    
	h10  = -1/D2*(u2-Mubar*h)* s1

        
      FUd = h7*exp(-h*LAI)+h8*exp(h*LAI)
      FQd = h9*exp(-h*LAI)+h10*exp(h*LAI)

c###############
   
C	Net radiation at snow surface below canopy (shading)
		Qsns = (Qsib*FQb+Qsib*FQd)*(1-A)

 
C	Net radiation reflected back from the  canopy and ground
            Qsnr = (Qsib*FUb+Qsid*FUd)         
								    
C	Net radiation  canopy
		Qsnc = Qsid+Qsib - Qsnr-Qsns 

c	ENDIF
6	RETURN
	END

c***********************************   NEW  LONGWAVE RADIATION MODEL- TWO STREAM  **************************  

	SUBROUTINE NETLONGRAD4(RH,Ta,Tss,Tc,Tk,Fs,EmC,EmS,SBC,cf,Taud1,Qli,
     *	                       Qlis,Qlns,Qlnc )                     !  Output: Qsns,Qsnc 
 
	REAL RH,Ta,Tss,Tc,EmC,EmS,SBC,Fs
	REAL EmA,ea, Qlis, Qlns,Qlnc
	REAL Qli, Qle,Qlcd,Qlcu,f1i,f2i,f3i,f1e,f2e,f3e,f1cd,f2cd,f3cd
	
c	ea   = SVPW(TA)*RH					            	! Air vapor pressure 
      TAK  = TA + TK
	Tssk = Tss +Tk  
      EmCS = EmC*(1-Fs)+EmS*Fs

	Tck  = Tc+Tk

c       Ems =1                          ! To compare with original model
	  Qle  = EmS*SBC*Tssk**4
	  Qlcd = (1-Taud1)*EmC*SBC*Tck**4
	  Qlcu = (1-Taud1)*EmCs*SBC*Tck**4


       f1i = Ems*Taud1/(1-(1-Emc)*(1-Taud1)*(1-Ems))
       f3i = (1-Taud1)*(1-EmCs)+Taud1*Taud1*(1-Ems)/
     *	     (1-(1-Emc)*(1-Taud1)*(1-Ems))
       f2i = 1-f1i-f3i 
       ! ********************
       f1e = Ems*(1-Emc)*(1-Taud1)/(1-(1-Emc)*(1-Taud1)*(1-Ems))
       f3e = Taud1/(1-(1-Emc)*(1-Taud1)*(1-Ems))
       f2e = 1-f1e-f3e 
       ! ********************
       f1cd = Ems/(1-(1-Emc)*(1-Taud1)*(1-Ems))
       f3cd = Taud1*(1-Ems)/(1-(1-Emc)*(1-Taud1)*(1-Ems))
       f2cd = 1-f1cd-f3cd 

CNet sub canopy longwave radiation
       Qlns = f1i*Qli+f1e*Qle+f1cd*Qlcd-Qle
CNet reflected longwave radiation rom canopy
      Qlnr = f3i*Qli+f3e*Qle+f3cd*Qlcd+Qlcu
CNet canopy longwave radiation
      Qlnc = f2i*Qli+ f2e*Qle+f2cd*Qlcd-Qlcu-Qlcd

********************************************************************** The other way to estimate the longwave radiation -finished

C     Net canopy longwave radiation

	IF (Taud1.EQ.1.0)THEN                  ! For LAI =0
      Qlnc =0.                                 ! Incase Qlnc has some value because of rounding error
               
	ENDIF
      RETURN
      END

